@@ -17,7 +17,7 @@
    limitations under the License.
 */
 
-/* $Id: rivetCore.c 1599307 2014-06-02 18:57:20Z mxmanghi $ */
+/* $Id: rivetCore.c 1650414 2015-01-08 23:03:12Z mxmanghi $ */
 
 /* Rivet config */
 #ifdef HAVE_CONFIG_H
@@ -359,13 +359,23 @@ TCL_CMD_HEADER( Rivet_Headers )
         Tcl_WrongNumArgs(interp, 1, objv, "option arg ?arg ...?");
         return TCL_ERROR;
     }
+
+    opt = Tcl_GetStringFromObj(objv[1], NULL);
+
+    /* Basic introspection returning the value of the headers_printed flag */
+
+    if (!strcmp("sent",opt))
+    {
+        Tcl_SetObjResult(interp, Tcl_NewIntObj(globals->req->headers_printed));
+        return TCL_OK;
+    }
+
     if (globals->req->headers_printed != 0)
     {
         Tcl_AddObjErrorInfo(interp,
                             "Cannot manipulate headers - already sent", -1);
         return TCL_ERROR;
     }
-    opt = Tcl_GetStringFromObj(objv[1], NULL);
 
     if (!strcmp("redirect", opt)) /* ### redirect ### */
     {
@@ -379,6 +389,19 @@ TCL_CMD_HEADER( Rivet_Headers )
         TclWeb_SetStatus(301, globals->req);
         return TCL_OK;
     }
+    else if (!strcmp("get", opt)) /* ### get ### */
+    {
+        const char* header_value;
+
+        if (objc != 3)
+        {
+            Tcl_WrongNumArgs(interp, 2, objv, "headername");
+            return TCL_ERROR;
+        }
+        header_value = TclWeb_OutputHeaderGet(Tcl_GetString(objv[2]),globals->req); 
+
+        Tcl_SetObjResult(interp,Tcl_NewStringObj(header_value ? header_value : "",-1));
+    }
     else if (!strcmp("set", opt)) /* ### set ### */
     {
         if (objc != 4)
